@@ -15,6 +15,7 @@
 
 """Common client utilities"""
 
+import copy
 import getpass
 import logging
 import os
@@ -27,6 +28,67 @@ from osc_lib import exceptions
 from osc_lib.i18n import _
 
 
+LOG = logging.getLogger(__name__)
+
+
+def backward_compat_col_lister(column_headers, columns, column_map):
+    """Convert the column headers to keep column backward compatibility.
+
+    Replace the new column name of column headers by old name, so that
+    the column headers can continue to support to show the old column name by
+    --column/-c option with old name, like: volume list -c 'Display Name'
+
+    :param column_headers: The column headers to be output in list command.
+    :param columns: The columns to be output.
+    :param column_map: The key of map is old column name, the value is new
+            column name, like: {'old_col': 'new_col'}
+    """
+    if not columns:
+        return column_headers
+    # NOTE(RuiChen): column_headers may be a tuple in some code, like:
+    #                volume v1, convert it to a list in order to change
+    #                the column name.
+    column_headers = list(column_headers)
+    for old_col, new_col in six.iteritems(column_map):
+        if old_col in columns:
+            LOG.warning(_('The column "%(old_column)s" was deprecated, '
+                          'please use "%(new_column)s" replace.') % {
+                              'old_column': old_col,
+                              'new_column': new_col}
+                        )
+            if new_col in column_headers:
+                column_headers[column_headers.index(new_col)] = old_col
+    return column_headers
+
+
+def backward_compat_col_showone(show_object, columns, column_map):
+    """Convert the output object to keep column backward compatibility.
+
+    Replace the new column name of output object by old name, so that
+    the object can continue to support to show the old column name by
+    --column/-c option with old name, like: volume show -c 'display_name'
+
+    :param show_object: The object to be output in create/show commands.
+    :param columns: The columns to be output.
+    :param column_map: The key of map is old column name, the value is new
+        column name, like: {'old_col': 'new_col'}
+    """
+    if not columns:
+        return show_object
+
+    show_object = copy.deepcopy(show_object)
+    for old_col, new_col in six.iteritems(column_map):
+        if old_col in columns:
+            LOG.warning(_('The column "%(old_column)s" was deprecated, '
+                          'please use "%(new_column)s" replace.') % {
+                              'old_column': old_col,
+                              'new_column': new_col}
+                        )
+            if new_col in show_object:
+                show_object.update({old_col: show_object.pop(new_col)})
+    return show_object
+
+
 def build_kwargs_dict(arg_name, value):
     """Return a dictionary containing `arg_name` if `value` is set."""
     kwargs = {}
@@ -118,7 +180,8 @@ def find_resource(manager, name_or_id, **kwargs):
     #                 of client exceptions.
     except Exception as ex:
         if (type(ex).__name__ == 'NotFound' or
-                type(ex).__name__ == 'HTTPNotFound'):
+                type(ex).__name__ == 'HTTPNotFound' or
+                type(ex).__name__ == 'TypeError'):
             pass
         else:
             raise
@@ -192,12 +255,40 @@ def format_dict(data):
     :rtype: a string formatted to key='value'
     """
 
+    if data is None:
+        return None
+
     output = ""
     for s in sorted(data):
         output = output + s + "='" + six.text_type(data[s]) + "', "
     return output[:-2]
 
 
+def format_dict_of_list(data, separator='; '):
+    """Return a formatted string of key value pair
+
+    :param data: a dict, key is string, value is a list of string, for example:
+                 {u'public': [u'2001:db8::8', u'172.24.4.6']}
+    :param separator: the separator to use between key/value pair
+                      (default: '; ')
+    :return: a string formatted to {'key1'=['value1', 'value2']} with separated
+             by separator
+    """
+    if data is None:
+        return None
+
+    output = []
+    for key in sorted(data):
+        value = data[key]
+        if value is None:
+            continue
+        value_str = format_list(value)
+        group = "%s=%s" % (key, value_str)
+        output.append(group)
+
+    return separator.join(output)
+
+
 def format_list(data, separator=', '):
     """Return a formatted strings
 
@@ -205,6 +296,8 @@ def format_list(data, separator=', '):
     :param separator: the separator to use between strings (default: ', ')
     :rtype: a string formatted based on separator
     """
+    if data is None:
+        return None
 
     return separator.join(sorted(data))
 
@@ -215,10 +308,42 @@ def format_list_of_dicts(data):
     :param data: a list of dicts
     :rtype: a string formatted to key='value' with dicts separated by new line
     """
+    if data is None:
+        return None
 
     return '\n'.join(format_dict(i) for i in data)
 
 
+def format_size(size):
+    """Display size of a resource in a human readable format
+
+    :param string size:
+        The size of the resource in bytes.
+
+    :returns:
+        Returns the size in human-friendly format
+    :rtype string:
+
+    This function converts the size (provided in bytes) of a resource
+    into a human-friendly format such as K, M, G, T, P, E, Z
+    """
+
+    suffix = ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']
+    base = 1000.0
+    index = 0
+
+    if size is None:
+        size = 0
+    while size >= base:
+        index = index + 1
+        size = size / base
+
+    padded = '%.1f' % size
+    stripped = padded.rstrip('0').rstrip('.')
+
+    return '%s%s' % (stripped, suffix[index])
+
+
 def get_client_class(api_name, version, version_map):
     """Returns the client class for the requested API version
 
@@ -267,7 +392,7 @@ def get_dict_properties(item, fields, mixed_case_fields=None, formatters=None):
         else:
             field_name = field.lower().replace(' ', '_')
         data = item[field_name] if field_name in item else ''
-        if field in formatters:
+        if field in formatters and data is not None:
             row.append(formatters[field](data))
         else:
             row.append(data)
@@ -320,7 +445,7 @@ def get_item_properties(item, fields, mixed_case_fields=None, formatters=None):
         else:
             field_name = field.lower().replace(' ', '_')
         data = getattr(item, field_name, '')
-        if field in formatters:
+        if field in formatters and data is not None:
             row.append(formatters[field](data))
         else:
             row.append(data)
